/***********************************************************************************************************************
 *
 * blueBill Core - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it
 * SCM: https://java.net/hg/bluebill~core-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.taxonomy;

import javax.annotation.Nonnull;

/***********************************************************************************************************************
 * 
 * A visitor for an {@link Taxonomy} which contains callback methods to navigate through the data.
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public interface TaxonomyVisitor
  {
    public static final Class<TaxonomyVisitor> TaxonomyVisitor = TaxonomyVisitor.class;
    
    /*******************************************************************************************************************
     *
     * Visits a {@link Taxon}. This method is called before visiting subtaxa (pre-order).
     * 
     * @param  taxon  the visited {@code Taxon}
     *
     ******************************************************************************************************************/
    public void preVisit (@Nonnull Taxon taxon);

    /*******************************************************************************************************************
     *
     * Visits a {@link Taxon}. This method is actually called just after {@link #preVisit()}, it makes sense to 
     * implement it when you don't need to distinguish between pre-order and post-order traversal.
     * 
     * @param  taxon  the visited {@code Taxon}
     *
     ******************************************************************************************************************/
    public void visit (@Nonnull Taxon taxon);

    /*******************************************************************************************************************
     *
     * Visits a {@link Taxon}. This method is called after visiting subtaxa (post-order).
     * 
     * @param  taxon  the visited {@code Taxon}
     *
     ******************************************************************************************************************/
    public void postVisit (@Nonnull Taxon taxon);
  }
